"""This module is for the translation mapping data of the
following document:

Document file: about_handler_options_type.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# About the handler options' type": "# ハンドラのoptions引数の型について",
    ##################################################
    "This page explains the event handler `options` argument's type.": "このページではイベントハンドラの`options`引数の型について説明します。",  # noqa
    ##################################################
    "## The dictionary type is acceptable": "## options引数には辞書の型を受け付けることができます",
    ##################################################
    "Each handler's `options` argument can accept a dictionary type value, like the following:": "以下のコードサンプルのように`options`引数は辞書の型の値を受け付けることができます。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing import Dict\n\nimport apysc as ap\n\n\ndef on_timer(e: ap.TimerEvent, options: Dict[str, str]) -> None:\n    """\n    The handler that a timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace(options["msg"])\n\n\ntimer: ap.Timer = ap.Timer(on_timer, delay=1000, options={"msg": "Hello!"})\ntimer.start()\n```': '```py\n# runnable\nfrom typing import Dict\n\nimport apysc as ap\n\n\ndef on_timer(e: ap.TimerEvent, options: Dict[str, str]) -> None:\n    """\n    The handler that a timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace(options["msg"])\n\n\ntimer: ap.Timer = ap.Timer(on_timer, delay=1000, options={"msg": "Hello!"})\ntimer.start()\n```',  # noqa
    ##################################################
    "## About the TypedDict annotation": "## TypedDictを使った型アノテーションについて",
    ##################################################
    "Sometimes using the `TypedDict` type annotation instead of the `dict` type annotation is helpful and makes it easy to read the code. The apysc check a handler options annotation and the actual options value type when you use the `TypedDict`\\.": "通常の辞書の代わりに`TypedDict`を使った型アノテーションは可読性の向上に役立つことがあります。apyscではoptions引数にTypedDictによる型アノテーションをサポートしており、実際に渡したoptionsの辞書の型とハンドラのTypedDictによるアノテーション間で型チェックを行うことができます。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _MsgOptions(TypedDict):\n    msg: str\n\n\ndef on_timer(e: ap.TimerEvent, options: _MsgOptions) -> None:\n    """\n    The handler that a timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace(options["msg"])\n\n\noptions: _MsgOptions = {"msg": "Hello!"}\ntimer: ap.Timer = ap.Timer(on_timer, delay=1000, options=options)\ntimer.start()\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _MsgOptions(TypedDict):\n    msg: str\n\n\ndef on_timer(e: ap.TimerEvent, options: _MsgOptions) -> None:\n    """\n    The handler that a timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace(options["msg"])\n\n\noptions: _MsgOptions = {"msg": "Hello!"}\ntimer: ap.Timer = ap.Timer(on_timer, delay=1000, options=options)\ntimer.start()\n```',  # noqa
    ##################################################
    "Notes: if you are using a Python 3.8 or later version, then importing the `TypedDict` from the `typing` package instead of the `typing_extensions` is available (e.g., `from typing import TypedDict`).": "特記事項: もしPython3.8もしくはそれ以降のPythonバージョンをお使いの場合には`typing_extensions`パッケージの代わりに`typing`パッケージから`TypedDict`をimportすることができます（例 : `from typing import TypedDict`）。",  # noqa
}



"""Variable name-related Validation interfaces.
"""

from typing import Any

from apysc._type.variable_name_mixin import VariableNameMixIn


def validate_variable_name_interface_type(
    *, instance: Any, additional_err_msg: str = ""
) -> VariableNameMixIn:
    """
    Validate specified instance type is VariableNameMixIn.

    Parameters
    ----------
    instance : *
        Instance to be checked.
    additional_err_msg : str, optional
        An additional error message to display.

    Returns
    -------
    instance : VariableNameMixIn
        Checked instance.

    Raises
    ------
    TypeError
        If specified instance type isn't VariableNameMixIn.
    """
    if not isinstance(instance, VariableNameMixIn):
        if additional_err_msg != "":
            additional_err_msg = f"\n{additional_err_msg}"
        raise TypeError(
            "Specified instance type is not VariableNameMixIn : "
            f"{type(instance)}{additional_err_msg}"
        )
    return instance



"""This module is for the handler interfaces'
validation implementations.
"""

import inspect
from inspect import Signature
from typing import Any
from typing import Callable
from typing import List


def validate_options_type(*, options: Any, additional_err_msg: str = "") -> None:
    """
    Validate a specified options type.

    Parameters
    ----------
    options : Any
        Target options value.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    TypeError
        If a specified options type is not the dictionary or None.
    """
    if options is None:
        return
    if isinstance(options, dict):
        return
    if additional_err_msg != "":
        additional_err_msg = f"\n{additional_err_msg}"
    raise TypeError(
        f"Handler's options argument must be a dictionary: {type(options)}"
        f"\n{options}{additional_err_msg}"
    )


def validate_handler_args_num(
    *, handler: Callable, additional_err_msg: str = ""
) -> None:
    """
    Validate specified handler's arguments number.

    Parameters
    ----------
    handler : Callable
        A target handler to validate.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        - If handler's arguments number is not 2.
    TypeError
        - If a specified handler is not callable.
    """
    if not callable(handler):
        raise TypeError(
            "A specified handler's argument is not callable: " f"{type(handler)}"
        )
    signature: Signature = inspect.signature(obj=handler)
    args_num: int = 0
    skipping_arg_names: List[str] = ["*", "**", "self", "cls"]
    arg_names: List[str] = []
    for parameter in signature.parameters:
        if parameter in skipping_arg_names:
            continue
        args_num += 1
        arg_names.append(parameter)
    if args_num != 2:
        if additional_err_msg != "":
            additional_err_msg = f"\n{additional_err_msg}"
        raise ValueError(
            "A specified handler's arguments number must be 2 "
            f"(actual: {args_num})"
            f"\nTarget argument names: {arg_names}"
            "\n\nThe first argument becomes event instance and the second "
            "one becomes the handler's option parameters."
            f"{additional_err_msg}"
        )



"""Implementations of the Line class.
"""

from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._display import graphics
from apysc._display.child_mixin import ChildMixIn
from apysc._display.graphics_base import GraphicsBase
from apysc._display.line_caps import LineCaps
from apysc._display.line_dash_dot_setting import LineDashDotSetting
from apysc._display.line_dash_setting import LineDashSetting
from apysc._display.line_dot_setting import LineDotSetting
from apysc._display.line_round_dot_setting import LineRoundDotSetting
from apysc._display.x_mixin import XMixIn
from apysc._display.y_mixin import YMixIn
from apysc._geom import point2d
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class Line(XMixIn, YMixIn, GraphicsBase, VariableNameSuffixMixIn):
    """
    The line vector graphics class.

    References
    ----------
    - Line class
        - https://simon-ritchie.github.io/apysc/en/line.html
    - Graphics draw_line interface
        - https://simon-ritchie.github.io/apysc/en/graphics_draw_line.html  # noqa
    - Graphics draw_dotted_line interface
        - https://simon-ritchie.github.io/apysc/en/graphics_draw_dotted_line.html  # noqa
    - Graphics draw_dashed_line interface
        - https://simon-ritchie.github.io/apysc/en/graphics_draw_dashed_line.html  # noqa
    - Graphics draw_round_dotted_line interface
        - https://simon-ritchie.github.io/apysc/en/graphics_draw_round_dotted_line.html  # noqa
    - Graphics draw_dash_dotted_line interface
        - https://simon-ritchie.github.io/apysc/en/graphics_draw_dash_dotted_line.html # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.line_style(color="#fff", thickness=5)
    >>> line: ap.Line = sprite.graphics.draw_line(
    ...     x_start=50, y_start=50, x_end=150, y_end=50
    ... )
    >>> line.line_color
    String('#ffffff')

    >>> line.line_thickness
    Int(5)
    """

    _start_point: "point2d.Point2D"
    _end_point: "point2d.Point2D"

    # self
    @arg_validation_decos.multiple_line_settings_are_not_set(arg_position_index=0)
    # start_point
    @arg_validation_decos.is_point_2d(arg_position_index=1)
    # end_point
    @arg_validation_decos.is_point_2d(arg_position_index=2)
    # line_color
    @arg_validation_decos.is_hex_color_code_format(arg_position_index=3)
    # line_alpha
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=4)
    # line_thickness
    @arg_validation_decos.is_integer(arg_position_index=5)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=5)
    # line_cap
    @arg_validation_decos.is_line_cap(arg_position_index=6, optional=True)
    # line_dot_setting
    @arg_validation_decos.is_line_dot_setting(arg_position_index=7)
    # line_dash_setting
    @arg_validation_decos.is_line_dash_setting(arg_position_index=8)
    # line_round_dot_setting
    @arg_validation_decos.is_line_round_dot_setting(arg_position_index=9)
    # line_dash_dot_setting
    @arg_validation_decos.is_line_dash_dot_setting(arg_position_index=10)
    # parent
    @arg_validation_decos.is_display_object_container(
        arg_position_index=11, optional=True
    )
    # variable_name_suffix
    @arg_validation_decos.is_builtin_string(arg_position_index=12, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        *,
        start_point: "point2d.Point2D",
        end_point: "point2d.Point2D",
        line_color: Union[str, String] = "",
        line_alpha: Union[float, Number] = 1.0,
        line_thickness: Union[int, Int] = 1,
        line_cap: Optional[Union[String, LineCaps]] = None,
        line_dot_setting: Optional[LineDotSetting] = None,
        line_dash_setting: Optional[LineDashSetting] = None,
        line_round_dot_setting: Optional[LineRoundDotSetting] = None,
        line_dash_dot_setting: Optional[LineDashDotSetting] = None,
        parent: Optional[ChildMixIn] = None,
        variable_name_suffix: str = "",
    ) -> None:
        """
        Create a line vector graphic.

        Parameters
        ----------
        start_point : Points2D
            Line start point.
        end_point : Points2D
            Line end point.
        line_color : str or String, default ''
            A line-color to set.
        line_alpha : float or Number, default 1.0
            A line-alpha to set.
        line_thickness : int or Int, default 1
            A line-thickness (line-width) to set.
        line_cap : String or LineCaps or None, default None
            A line-cap setting to set.
        line_dot_setting : LineDotSetting or None, default None
            A dot setting to set.
        line_dash_setting : LineDashSetting or None, default None
            A dash setting to set.
        line_round_dot_setting : LineRoundDotSetting or None, default None
            A round-dot setting to set.
        line_dash_dot_setting : LineDashDotSetting or None, default None
            A dash dot (1-dot chain) setting to set.
        parent : ChildMixIn or None, default None
            A parent instance to add this instance.
            If a specified value is None, this interface uses
            a stage instance.
        variable_name_suffix : str, default ''
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript's debugging.

        References
        ----------
        - Line class
            - https://simon-ritchie.github.io/apysc/en/line.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> line: ap.Line = ap.Line(
        ...     start_point=ap.Point2D(x=50, y=50),
        ...     end_point=ap.Point2D(x=150, y=50),
        ...     line_color="#ffffff",
        ...     line_thickness=3,
        ... )
        >>> line.line_color
        String('#ffffff')
        >>> line.line_thickness
        Int(3)
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names

        self._variable_name_suffix = variable_name_suffix
        variable_name: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.LINE
        )
        self.variable_name = variable_name
        self._set_initial_basic_values(
            fill_color="",
            fill_alpha=1,
            line_color=line_color,
            line_thickness=line_thickness,
            line_alpha=line_alpha,
            line_cap=line_cap,
            line_joints=None,
        )
        self._start_point = start_point
        self._end_point = end_point
        self._append_constructor_expression()
        self._set_line_setting_if_not_none_value_exists(
            line_dot_setting=line_dot_setting,
            line_dash_setting=line_dash_setting,
            line_round_dot_setting=line_round_dot_setting,
            line_dash_dot_setting=line_dash_dot_setting,
        )
        self._set_initial_x_and_y_with_minimum_point()
        super(Line, self).__init__(parent=parent, variable_name=variable_name)

    @final
    def _set_initial_x_and_y_with_minimum_point(self) -> None:
        """
        Set initial x and y properties coordinate with
        a minimum point.
        """
        min_x: int = min(self._start_point._x._value, self._end_point._x._value)
        min_y: int = min(self._start_point._y._value, self._end_point._y._value)
        self._x = Int(min_x)
        self._y = Int(min_y)

    @classmethod
    @final
    def _create_with_graphics(
        cls,
        *,
        graphics: "graphics.Graphics",
        start_point: "point2d.Point2D",
        end_point: "point2d.Point2D",
        variable_name_suffix: str = "",
    ) -> "Line":
        """
        Create a line instance with the instance of
        specified graphics.

        Parameters
        ----------
        graphics : Graphics
            Graphics instance to link this instance.
        start_point : Points2D
            Line start point.
        end_point : Points2D
            Line end point.
        variable_name_suffix : str, default ''
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript's debugging.

        Returns
        -------
        line : Line
            A created line instance.
        """
        line: Line = Line(
            start_point=start_point,
            end_point=end_point,
            line_color=graphics._line_color,
            line_alpha=graphics._line_alpha,
            line_thickness=graphics._line_thickness,
            line_cap=graphics._line_cap,
            line_dot_setting=graphics._line_dot_setting,
            line_dash_setting=graphics._line_dash_setting,
            line_round_dot_setting=graphics._line_round_dot_setting,
            line_dash_dot_setting=graphics._line_dash_dot_setting,
            parent=graphics,
            variable_name_suffix=variable_name_suffix,
        )
        return line

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_constructor_expression(self) -> None:
        """
        Append a constructor expression.
        """
        import apysc as ap

        stage: ap.Stage = ap.get_stage()
        points_str: str = self._make_points_expression()
        expression: str = (
            f"var {self.variable_name} = {stage.variable_name}"
            f"\n  .line({points_str})"
            "\n  .attr({"
        )
        expression = self._append_basic_vals_expression(
            expression=expression, indent_num=2
        )
        expression += "\n  });"
        ap.append_js_expression(expression=expression)

    @final
    def _make_points_expression(self) -> str:
        """
        Make line start and end expression str.

        Returns
        -------
        expression : str
            Each point expression.
        """
        import apysc as ap

        start_point: ap.Point2D = self._start_point
        end_point: ap.Point2D = self._end_point
        expression: str = (
            f"{start_point.x.variable_name}, "
            f"{start_point.y.variable_name}, "
            f"{end_point.x.variable_name}, "
            f"{end_point.y.variable_name}"
        )
        return expression

    def __repr__(self) -> str:
        """
        Get a string representation of this instance (for the sake of
        debugging).

        Returns
        -------
        repr_str : str
            Type name and variable name will be set
            (e.g., `Line('<variable_name>')`).
        """
        repr_str: str = f"Line('{self.variable_name}')"
        return repr_str



"""Class implementation for the attribute linking mix-in.

This mix-in updates an old property value to achieve
consistency in the handler functions.
"""

from typing import Dict
from typing import List
from typing import Union

from typing_extensions import final

from apysc._type.boolean import Boolean
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String

_AttrName = str
_Attr = Union[Int, Number, String, Boolean]


class AttrLinkingMixIn:

    _attr_linking_stack: Dict[_AttrName, List[_Attr]]

    @final
    def _initialize_attr_linking_stack(self, *, attr_name: str) -> None:
        """
        Initialize the _attr_linking_stack attribute if
        this instance does not initialize it yet.

        Parameters
        ----------
        attr_name : str
            Target attribute name.
        """
        if not hasattr(self, "_attr_linking_stack"):
            self._attr_linking_stack = {}
        if attr_name in self._attr_linking_stack:
            return
        self._attr_linking_stack[attr_name] = []

    @final
    def _append_attr_to_linking_stack(self, *, attr: _Attr, attr_name: str) -> None:
        """
        Append an attribute to the linking attribute stack.

        Parameters
        ----------
        attr : Int or Number or String or Boolean
            Target attribute to be appended.
        attr_name : str
            Target attribute name.
        """
        self._initialize_attr_linking_stack(attr_name=attr_name)
        if self._is_target_attr_already_linked(attr=attr, attr_name=attr_name):
            return
        self._attr_linking_stack[attr_name].append(attr)

    @final
    def _is_target_attr_already_linked(self, *, attr: _Attr, attr_name: str) -> bool:
        """
        Get a boolean value whether this instance already
        appends a specified attribute to the linking
        attribute stack.

        Parameters
        ----------
        attr : Int or Number or String or Boolean
            Target attribute to be appended.
        attr_name : str
            Target attribute name.

        Returns
        -------
        result : bool
            If this instance already appends a specified
            attribute to the linking stack, this interface
            returns True.
        """
        self._initialize_attr_linking_stack(attr_name=attr_name)
        for in_stack_value in self._attr_linking_stack[attr_name]:
            if in_stack_value.variable_name == attr.variable_name:
                return True
        return False

    @final
    def _append_applying_new_attr_val_exp(
        self, *, new_attr: _Attr, attr_name: str
    ) -> None:
        """
        Append the expression of applying a new attribute
        value to each stacked value.

        Parameters
        ----------
        new_attr : Int or Number or String or Boolean
            New attribute value.

        attr_name : str
            Target attribute name.
        """
        import apysc as ap

        self._initialize_attr_linking_stack(attr_name=attr_name)
        if not self._attr_linking_stack:
            return
        new_attr_name: str = new_attr.variable_name
        expression: str = ""
        for stacked_value in self._attr_linking_stack[attr_name]:
            if stacked_value.variable_name == new_attr_name:
                continue
            if expression != "":
                expression += "\n"
            expression += f"{stacked_value.variable_name} = {new_attr_name};"
        ap.append_js_expression(expression=expression)



"""Class implementation for line alpha mix-in.
"""

from typing import Dict
from typing import Union

from typing_extensions import final

from apysc._animation.animation_line_alpha_mixin import AnimationLineAlphaMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.number import Number
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_suffix_attr_mixin import VariableNameSuffixAttrMixIn
from apysc._validation import arg_validation_decos


class LineAlphaMixIn(
    VariableNameSuffixAttrMixIn,
    AnimationLineAlphaMixIn,
    RevertMixIn,
    AttrLinkingMixIn,
):

    _line_alpha: Number

    @final
    def _initialize_line_alpha_if_not_initialized(self) -> None:
        """
        Initialize _line_alpha attribute if this interface does not
        initialize it yet.
        """
        if hasattr(self, "_line_alpha"):
            return
        suffix: str = self._get_attr_variable_name_suffix(attr_identifier="line_alpha")
        self._line_alpha = Number(
            1.0,
            variable_name_suffix=suffix,
            skip_init_substitution_expression_appending=True,
        )

        self._append_line_alpha_attr_linking_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_line_alpha_attr_linking_setting(self) -> None:
        """
        Append a line alpha attribute linking settings.
        """
        self._append_applying_new_attr_val_exp(
            new_attr=self._line_alpha, attr_name="line_alpha"
        )
        self._append_attr_to_linking_stack(
            attr=self._line_alpha, attr_name="line_alpha"
        )

    @property
    @add_debug_info_setting(module_name=__name__)
    def line_alpha(self) -> Number:
        """
        Get this instance's line alpha (opacity).

        Returns
        -------
        line_alpha : Number
            Current line alpha (opacity. 0.0 to 1.0).

        References
        ----------
        - GraphicsBase line_alpha interface
            - https://simon-ritchie.github.io/apysc/en/graphics_base_line_alpha.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color="#fff", thickness=5, alpha=1.0)
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> rectangle.line_alpha = ap.Number(0.5)
        >>> rectangle.line_alpha
        Number(0.5)
        """
        from apysc._type import value_util

        self._initialize_line_alpha_if_not_initialized()
        return value_util.get_copy(value=self._line_alpha)

    @line_alpha.setter
    @arg_validation_decos.is_apysc_num(arg_position_index=1)
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def line_alpha(self, value: Number) -> None:
        """
        Update this instance's line alpha (opacity).

        Parameters
        ----------
        value : Number
            Line alpha (opacity) to set.

        References
        ----------
        - GraphicsBase line_alpha interface
            - https://simon-ritchie.github.io/apysc/en/graphics_base_line_alpha.html  # noqa
        """
        self._initialize_line_alpha_if_not_initialized()
        self._update_line_alpha_and_skip_appending_exp(value=value)
        self._line_alpha._append_incremental_calc_substitution_expression()
        self._append_line_alpha_update_expression()

        self._append_line_alpha_attr_linking_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_line_alpha_update_expression(self) -> None:
        """
        Append line alpha updating expression.
        """
        import apysc as ap
        from apysc._type import value_util

        line_alpha_str: str = value_util.get_value_str_for_expression(
            value=self._line_alpha
        )
        expression: str = f"{self.variable_name}.stroke({{opacity: {line_alpha_str}}});"
        ap.append_js_expression(expression=expression)

    @final
    def _update_line_alpha_and_skip_appending_exp(
        self, *, value: Union[float, Number]
    ) -> None:
        """
        Update line alpha and skip appending expression.

        Parameters
        ----------
        value : float or Number
            Line alpha (opacity) to set.
        """
        import apysc as ap

        if isinstance(value, ap.Number):
            value_: ap.Number = value
        else:
            suffix: str = self._get_attr_variable_name_suffix(
                attr_identifier="line_alpha"
            )
            value_ = Number(value, variable_name_suffix=suffix)
        self._line_alpha = value_

    _line_alpha_snapshots: Dict[str, float]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make value's snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_line_alpha_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_line_alpha_snapshots",
            value=self._line_alpha._value,
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert value if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._line_alpha._value = self._line_alpha_snapshots[snapshot_name]



"""Class implementation for the enter frame mix-in.
"""

from typing import Callable
from typing import Dict
from typing import Optional
from typing import TypeVar

from typing_extensions import TypedDict
from typing_extensions import final

from apysc._event.enter_frame_event import EnterFrameEvent
from apysc._event.handler import HandlerData
from apysc._event.set_handler_data_mixin import SetHandlerDataMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._time.fps import FPS
from apysc._type.boolean import Boolean
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._validation import arg_validation_decos

_Options = TypeVar("_Options")
_Handlername = str


class _HandlerSettings(TypedDict):
    handler_name: str
    is_stopped: Boolean
    fps: Int
    fps_interval: Number


class EnterFrameMixIn(SetHandlerDataMixIn[EnterFrameEvent]):

    _enter_frame_handlers: Dict[str, HandlerData[EnterFrameEvent]]
    _enter_frame_handler_settings: Dict[_Handlername, _HandlerSettings]

    @final
    @arg_validation_decos.handler_args_num(arg_position_index=1)
    @arg_validation_decos.is_fps(arg_position_index=2)
    @arg_validation_decos.handler_options_type(arg_position_index=3)
    @add_debug_info_setting(module_name=__name__)
    def enter_frame(
        self,
        handler: Callable[[EnterFrameEvent, _Options], None],
        *,
        fps: FPS = FPS.FPS_60,
        options: Optional[_Options] = None,
    ) -> None:
        """
        Add an enter frame event listener setting.

        Parameters
        ----------
        handler : Callable[[EnterFrameEvent, _Options], None]
            An handler function to handle the enter frame event.
        fps : FPS, default FPS.FPS_60
            Frame per second to set.
        options : Optional[_Options], optional
            Optional arguments to pass to a handler function.
        """

        self._initialize_enter_frame_handlers_if_not_initialized()
        self._set_handler_data(
            handler=handler,
            handlers_dict=self._enter_frame_handlers,
            options=options,
        )

    def _initialize_enter_frame_handlers_if_not_initialized(self) -> None:
        """
        Initialize the `_enter_frame_handlers` attribute if this interface
        has not initialized it yet.
        """
        if hasattr(self, "_enter_frame_handlers"):
            return
        self._enter_frame_handlers = {}



"""This module is for the translation mapping data of the
following document:

Document file: contains.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# contains interface": "# contains インターフェイス",
    ##################################################
    "This page explains the container class, like the `Graphics`\\, `Sprite`\\, `Stage`) `contains` method interface.": "このページでは`Graphics`や`Sprite`、`Stage`などのコンテナとしての各クラスが持つ`contains`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `contains` interface returns the boolean (`Boolean`) value whether a container instance has a given child or not.": "`contains`インターフェイスは引数に指定された子のインスタンスを対象のコンテナが持つかどうかの真偽値（`Boolean`）を返却します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The following example checks whether the first rectangle is a child of the `Sprite` container. If it is true, remove the rectangle from the sprite and display a log to the console (please press F12 to display that message).": "以下のコード例では最初の四角が`Sprite`のコンテナーの子かどうかをチェックしています。もしその子を含んでいればその子を取り除き、ブラウザのコンソール上にログを表示しています（ログの表示にはF12キーを押してください）。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_sprite_click(e: ap.MouseEvent[ap.Sprite], options: _RectOptions) -> None:\n    """\n    The handler that the sprite calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    sprite: ap.Sprite = e.this\n    rectangle_1: ap.Rectangle = options["rectangle"]\n    condition: ap.Boolean = sprite.graphics.contains(child=rectangle_1)\n    with ap.If(condition):\n        sprite.remove_child(child=rectangle_1)\n        ap.trace("Removed the rectangle!")\n\n\nap.Stage(\n    background_color="#333", stage_width=250, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nsprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\noptions: _RectOptions = {"rectangle": rectangle_1}\nsprite.click(on_sprite_click, options=options)\n\nap.save_overall_html(dest_dir_path="sprite_contains_basic_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_sprite_click(e: ap.MouseEvent[ap.Sprite], options: _RectOptions) -> None:\n    """\n    The handler that the sprite calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    sprite: ap.Sprite = e.this\n    rectangle_1: ap.Rectangle = options["rectangle"]\n    condition: ap.Boolean = sprite.graphics.contains(child=rectangle_1)\n    with ap.If(condition):\n        sprite.remove_child(child=rectangle_1)\n        ap.trace("Removed the rectangle!")\n\n\nap.Stage(\n    background_color="#333", stage_width=250, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nsprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\noptions: _RectOptions = {"rectangle": rectangle_1}\nsprite.click(on_sprite_click, options=options)\n\nap.save_overall_html(dest_dir_path="sprite_contains_basic_usage/")\n```',  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [add_child and remove_child interfaces](add_child_and_remove_child.md)": "- [add_child （子の追加）と remove_child （子の削除）のインターフェイス](jp_add_child_and_remove_child.md)",  # noqa
    ##################################################
    "## contains API": "## contains API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a boolean whether this instance contains a specified child.<hr>": "指定された子のインスタンスを持っているかどうかの真偽値を取得します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `child`: DisplayObject": "- `child`: DisplayObject",
    ##################################################
    "  - Child instance to check.": "  - チェック対象の子のインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `result`: Boolean": "- `result`: Boolean",
    ##################################################
    "  - If this instance contains a specified child, this method returns True.": "  - このインスタンスが指定された子を持つ場合Trueが設定されます。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af", alpha=0.5)\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> sprite.graphics.contains(rectangle)\nBoolean(True)\n\n>>> rectangle.remove_from_parent()\n>>> sprite.graphics.contains(rectangle)\nBoolean(False)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af", alpha=0.5)\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> sprite.graphics.contains(rectangle)\nBoolean(True)\n\n>>> rectangle.remove_from_parent()\n>>> sprite.graphics.contains(rectangle)\nBoolean(False)\n```',  # noqa
}



"""Class implementation for the rotation_around_center_interface
interface.
"""

from typing import Dict

from typing_extensions import final

from apysc._animation.animation_rotation_around_center_mixin import (
    AnimationRotationAroundCenterMixIn,
)
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.int import Int
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_suffix_attr_mixin import VariableNameSuffixAttrMixIn
from apysc._validation import arg_validation_decos


class RotationAroundCenterMixIn(
    VariableNameSuffixAttrMixIn,
    AnimationRotationAroundCenterMixIn,
    RevertMixIn,
    AttrLinkingMixIn,
):

    _rotation_around_center: Int

    @final
    def _initialize_rotation_around_center_if_not_initialized(self) -> None:
        """
        Initialize the `_rotation_around_center` attribute if this interface
        does not initialize it yet.
        """
        if hasattr(self, "_rotation_around_center"):
            return
        suffix: str = self._get_attr_variable_name_suffix(
            attr_identifier="rotation_around_center"
        )
        self._rotation_around_center = Int(
            0,
            variable_name_suffix=suffix,
            skip_init_substitution_expression_appending=True,
        )

        self._append_rotation_around_center_attr_linking_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_rotation_around_center_attr_linking_setting(self) -> None:
        """
        Append a rotation around a center attribute linking setting.
        """
        self._append_applying_new_attr_val_exp(
            new_attr=self._rotation_around_center, attr_name="rotation_around_center"
        )
        self._append_attr_to_linking_stack(
            attr=self._rotation_around_center, attr_name="rotation_around_center"
        )

    @property
    @add_debug_info_setting(module_name=__name__)
    def rotation_around_center(self) -> Int:
        """
        Get a rotation value around the center of this instance.

        Returns
        -------
        rotation_around_center : Int
            Rotation value around the center of this instance.

        References
        ----------
        - GraphicsBase rotation_around_center interface
            - https://simon-ritchie.github.io/apysc/en/graphics_base_rotation_around_center.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color="#0af")
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> rectangle.rotation_around_center = ap.Int(45)
        >>> rectangle.rotation_around_center
        Int(45)
        """
        from apysc._type import value_util

        self._initialize_rotation_around_center_if_not_initialized()
        return value_util.get_copy(value=self._rotation_around_center)

    @rotation_around_center.setter
    @arg_validation_decos.is_apysc_num(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def rotation_around_center(self, value: Int) -> None:
        """
        Update a rotation value around the center of this instance.

        Parameters
        ----------
        value : Int
            Rotation value around the center of this instance.

        References
        ----------
        - GraphicsBase rotation_around_center interface
            - https://simon-ritchie.github.io/apysc/en/graphics_base_rotation_around_center.html  # noqa
        """
        import apysc as ap

        self._initialize_rotation_around_center_if_not_initialized()
        before_value: ap.Int = self._rotation_around_center
        self._rotation_around_center = value
        self._append_rotation_around_center_update_expression(before_value=before_value)

        self._append_rotation_around_center_attr_linking_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_rotation_around_center_update_expression(
        self, *, before_value: Int
    ) -> None:
        """
        Append the rotation around the center of this instance
        updating expression.

        Parameters
        ----------
        before_value : ap.Int
            Before updating value.
        """
        import apysc as ap
        from apysc._type import value_util

        before_value_str: str = value_util.get_value_str_for_expression(
            value=before_value
        )
        after_value_str: str = value_util.get_value_str_for_expression(
            value=self._rotation_around_center
        )
        expression: str = (
            f"{self.variable_name}.rotate(-{before_value_str});"
            f"\n{self.variable_name}.rotate({after_value_str});"
            f"\n{before_value_str} = {after_value_str};"
        )
        ap.append_js_expression(expression=expression)

    _rotation_around_center_snapshots: Dict[str, int]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_rotation_around_center_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_rotation_around_center_snapshots",
            value=int(self._rotation_around_center._value),
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._rotation_around_center._value = self._rotation_around_center_snapshots[
            snapshot_name
        ]



"""This module is for the translation mapping data of the
following document:

Document file: save_overall_html.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# save_overall_html interface": "# save_overall_html インターフェイス",
    ##################################################
    "This page explains the `save_overall_html` function interface.": "このページでは`save_overall_html`関数のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `save_overall_html` function interface will export the overall HTML and JavaScript files. At the end of the apysc project, this function's calling is necessary to export the HTML.": "`save_overall_html`関数のインターフェイスはHTMLとJavaScript全体のファイルを出力します。apyscのプロジェクトの最後でHTMLなどを出力するためにこの関数の呼び出しが必要になります。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `save_overall_html` function need at least one argument, `dest_dir_path`. This argument determines a directory path to save HTML and JavaScript files.": "`save_overall_html`関数では最低限`dest_dir_path`引数の指定が必要になります。この引数はHTMLとJavaScriptの各ファイルの出力先のディレクトリとなります。",  # noqa
    ##################################################
    "The following code example exports the HTML and JavaScript files, and the exported HTML displays the blank stage (150 px width and height).": "以下のコード例ではHTMLとJavaScriptの各ファイル出力しています。出力したHTMLでは縦横150pxの空のステージのみ表示されるようにしてあります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nap.save_overall_html(dest_dir_path="save_overall_html_interface_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nap.save_overall_html(dest_dir_path="save_overall_html_interface_basic_usage/")\n```',  # noqa
    ##################################################
    "The preceding code exports the `save_overall_html_interface_basic_usage/index.html` and the other JavaScript library files.": "上記のコードでは`save_overall_html_interface_basic_usage/index.html`のパスにHTMLファイルや他のJavaScriptのライブラリファイルが出力されます。",  # noqa
    ##################################################
    "## Minify the HTML": "## HTMLを最小化する",
    ##################################################
    "The `save_overall_html` function has the `minify` optional argument (default is True). This interface minifies an output HTML if this value is `True`\\. The `False` setting is sometimes helpful for debugging.": "`save_overall_html`関数には`minify`のオプションの引数が存在します（デフォルトではTrueになっています）。この設定はもしTrueになっていきれば出力結果のHTMLを最小化（minify）します。Falseの設定はデバッグ時などに便利な時があります。",  # noqa
    ##################################################
    '```py\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nap.save_overall_html(dest_dir_path="dest_dir/", minify=False)\n```': '```py\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nap.save_overall_html(dest_dir_path="dest_dir/", minify=False)\n```',  # noqa
    ##################################################
    "## JavaScript libs directory path setting and skip option": "## JavaScriptのライブラリのディレクトリパスの設定と出力スキップの設定",  # noqa
    ##################################################
    "If you want to adjust the JavaScript library paths, set the `js_lib_dir_path` optional argument. This option overrides the JavaScript library paths in an exported HTML (`index.html`).": "もしもJavaScriptのライブラリパスを調整したい場合、`js_lib_dir_path`のオプションとなる引数で設定することができます。このオプションは出力されたHTML（`index.html`）内のJavaScriptのライブラリのパス指定を上書きします。",  # noqa
    ##################################################
    "This setting is sometimes helpful when you want to export the HTML with specified JavaScript library paths, for instance, the Django library static directory.": "この設定は例えばDjangoのようなライブラリの静的ファイルのディレクトリを指定する形でJavaScriptライブラリのパスを指定してHTMLを出力したい場合などに役立つ時があります。",  # noqa
    ##################################################
    "Also, the `skip_js_lib_exporting` option is helpful when you want to skip the already exported js files. This setting skips the JavaScript library exporting.": "また、`skip_js_lib_exporting`の引数のオプションも既に出力済みのJavaScriptライブラリの出力を省略したい場合に役立ちます。この設定を有効にするとJavaScriptライブラリのファイルは出力されなくなります。",  # noqa
    ##################################################
    '```py\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nap.save_overall_html(\n    dest_dir_path="dest_dir/", js_lib_dir_path="static/js/", skip_js_lib_exporting=True\n)\n```': '```py\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nap.save_overall_html(\n    dest_dir_path="dest_dir/", js_lib_dir_path="static/js/", skip_js_lib_exporting=True\n)\n```',  # noqa
    ##################################################
    "Notes: The `js_lib_dir_path` option does not change the js files exporting destination directory currently.": "特記事項: 現在の実装では`js_lib_dir_path`の設定ではJavaScriptファイルの出力先は変更されません。",  # noqa
    ##################################################
    "## Change the HTML file name by the html_file_name option": "## html_file_name オプションを使用してHTMLのファイル名を変更する",  # noqa
    ##################################################
    "If you need to change the output HTML file name, use the `html_file_name` optional argument. This argument changes the HTML file name from `index.html` to any other name.": "出力されるHTMLのファイル名を変更したい場合には`html_file_name`のオプションの引数を指定することで変更することができます。この引数はHTMLのファイル名を`index.html`から任意の他の名前に変更します。",  # noqa
    ##################################################
    '```py\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nap.save_overall_html(dest_dir_path="dest_dir/", html_file_name="chart.html")\n```': '```py\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nap.save_overall_html(dest_dir_path="dest_dir/", html_file_name="chart.html")\n```',  # noqa
    ##################################################
    "## Bundle each JavaScript library to the signle HTML file by the embed_js_libs option": "## embed_js_libs オプションでJavaScriptのライブラリを1つのHTMLファイルにまとめる",  # noqa
    ##################################################
    "You can bundle each JavaScript library to the single output HTML file by the `embed_js_libs` optional argument. This option is maybe useful when you need to pass the output file to the other members.": "`embed_js_libs`のオプションの引数を設定することで、出力結果のHTML内に各JavaScriptライブラリの内容を含めてまとめることができます。この設定は他のメンバーにファイルを共有する場合などに1つのファイルのみで扱うことができ便利な時があります。",  # noqa
    ##################################################
    '```py\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nap.save_overall_html(dest_dir_path="dest_dir/", embed_js_libs=True)\n```': '```py\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nap.save_overall_html(dest_dir_path="dest_dir/", embed_js_libs=True)\n```',  # noqa
    ##################################################
    "## Change the stdout setting by the verbose option": "## verbose オプションで標準出力の設定を変更する",  # noqa
    ##################################################
    "The `verbose` optional argument changes the exporting stdout behavior. If the specified value is 0, the apysc displays nothing. If 1 or the other values is specified, the apysc displays the stdout.": "`verbose`のオプションの引数はファイル出力時の標準出力の挙動を変更します。もしも0が指定された場合標準出力に何も表示しなくなります。1もしくは他の値を指定すればapyscライブラリは標準出力を表示します。",  # noqa
    ##################################################
    '```py\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nap.save_overall_html(dest_dir_path="dest_dir/", verbose=0)\n```': '```py\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nap.save_overall_html(dest_dir_path="dest_dir/", verbose=0)\n```',  # noqa
    ##################################################
    "## save_overall_html API": "## save_overall_html API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Save the overall HTML and js files under the specified directory path.<hr>": "指定されたディレクトリパス以下にHTMLとJavaScriptのファイル全体を出力します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `dest_dir_path`: str": "- `dest_dir_path`: str",
    ##################################################
    "  - Destination directory path to save each HTML and js file.": "  - 各HTMLとJavaScriptファイルの保存先となるディレクトリパス。",  # noqa
    ##################################################
    "- `html_file_name`: str, default 'index.html'": "- `html_file_name`: str, default 'index.html'",  # noqa
    ##################################################
    "  - The output HTML file name.": "  - 出力されるHTMLのファイル名。",
    ##################################################
    "- `minify`: bool, default True": "- `minify`: bool, default True",
    ##################################################
    "  - Boolean value indicates whether minify HTML and js or not. The False setting is helpful when debugging.": "  - HTMLとJavaScriptの内容を最小化（minify）するかどうかの真偽値。Falseの設定はデバッグ時などに便利な時があります。",  # noqa
    ##################################################
    "- `js_lib_dir_path`: str, default './'": "- `js_lib_dir_path`: str, default './'",  # noqa
    ##################################################
    "  - JavaScript libraries directory path. This setting applies to a JavaScript source path in HTML. If not specified, then set the same directory with HTML. This setting is maybe helpful to set js lib directory, such as Django's static (static_collected) directory. This interface recommends setting True value to the `skip_js_lib_exporting` argument if this argument sets.": "  - JavaScriptライブラリのパスの設定。この設定はHTML内のJavaScriptのコードのパスの指定部分に影響します。指定されていない場合にはHTMLと同じディレクトリが設定されます。この設定はDjangoのようなライブラリの静的ファイルのディレクトリを指定する場合などに便利なことがあります。もしこの引数が設定された場合には`skip_js_lib_exporting`の設定も有効にすることが推奨されます。",  # noqa
    ##################################################
    "- `skip_js_lib_exporting`: bool, default False": "- `skip_js_lib_exporting`: bool, default False",  # noqa
    ##################################################
    "  - If True, this interface does not export JavaScript libraries.": "  - Trueが設定された場合、このインターフェイスはJavaScriptの各ライブラリを出力しなくなります。",  # noqa
    ##################################################
    "- `embed_js_libs`: bool, default False": "- `embed_js_libs`: bool, default False",
    ##################################################
    "  - Option to embed the JavaScript libraries script to the output HTML or not. If True, the output HTML becomes enormous, and be only one HTML file. Occasionally, this option is useful when sharing the exported file or using the output file with an iframe tag to avoid the CORS error.": "  - 各JavaScriptライブラリを出力されるHTML内に埋め込むかどうかの設定です。もしTrueが設定された場合、出力されるHTMLは大きくなり、そして1つのHTMLファイルのみ出力されるようになります。この設定は出力されたファイルをiframeタグで使う際にCORSのエラーを回避したい時などに役立つことがあります。",  # noqa
    ##################################################
    "- `verbose`: int, default 1": "- `verbose`: int, default 1",
    ##################################################
    "  - The Logging setting. If 0 is specified, this interface does not display a logging message. If 1 or the other value is specified, this interface displays a message usually.": "  - ロギング（ログ表示）の設定です。0が指定された場合、このインターフェイスはログのメッセージを表示しなくなります。1もしくは他の値を指定した場合、このインターフェイスはログのメッセージを通常通り表示します。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    "This interface empties a specified directory before saving.<hr>": "このインターフェイスは指定された出力先のディレクトリを出力前に空にします。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> # Do something here...\n>>> ap.save_overall_html(dest_dir_path="tmp/output/")\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> # Do something here...\n>>> ap.save_overall_html(dest_dir_path="tmp/output/")\n```',  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: mouse_event_basic.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Basic mouse event interfaces": "# 基本的なマウスイベントの各インターフェイス",
    ##################################################
    "This page explains the basic mouse event interfaces, like the `this` attribute.": "このページでは`this`属性などのマウスイベントの基本的な各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## Basic binding usage": "## 基本的なイベント登録処理の使い方",
    ##################################################
    "Each mouse event binding interface accepts `handler` and `options` arguments. The `handler` argument is each interface's callable object when event dispatching.": "各マウスイベント設定のインターフェイスは`handler`と`options`引数を受け付けます。`handler`引数はイベントが発行された際に使用される関数などのオブジェクトです。",  # noqa
    ##################################################
    "The `options` argument is an optional parameter dictionary to be passed to the handler. You can skip this argument.": "`options`引数はハンドラへ渡される追加の任意の辞書のパラメーターです。この引数は省略できます。",  # noqa
    ##################################################
    "For instance, you can set the `click` event as follows:": "例えば`click`のイベントを以下のコードのように設定することができます:",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\nclass _ColorOptions(TypedDict):\n    color: str\n\n\ndef on_rectangle_click(e: ap.MouseEvent, options: _ColorOptions) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    # Change the clicked rectangle color to the passed color.\n    rectangle: ap.Rectangle = e.this\n    color: ap.String = ap.String(options["color"])\n    rectangle.fill_color = color\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\noptions: _ColorOptions = {"color": "#f0a"}\nrectangle.click(handler=on_rectangle_click, options=options)\n\nap.save_overall_html(dest_dir_path="mouse_event_basic_basic_binding_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\nclass _ColorOptions(TypedDict):\n    color: str\n\n\ndef on_rectangle_click(e: ap.MouseEvent, options: _ColorOptions) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    # Change the clicked rectangle color to the passed color.\n    rectangle: ap.Rectangle = e.this\n    color: ap.String = ap.String(options["color"])\n    rectangle.fill_color = color\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\noptions: _ColorOptions = {"color": "#f0a"}\nrectangle.click(handler=on_rectangle_click, options=options)\n\nap.save_overall_html(dest_dir_path="mouse_event_basic_basic_binding_usage/")\n```',  # noqa
    ##################################################
    "If you click the rectangle, the handler changes the rectangle color to the specified options color.": "四角をクリックした場合、ハンドラは四角の色をパラメーターに渡された色に変更します。",  # noqa
    ##################################################
    "There are many mouse events binding interfaces, such as the `click`\\, `mousedown`\\, `mouseup`\\, `mouseover`\\, `mouseout`\\, and `mousemove` that the `DisplayObject` instance has.": "`DisplayObject`の各インスタンスには`click`や`mousedown`、`mouseup`、`mouseover`、`mouseout`、`mousemove`などの様々なイベント設定用のインターフェイスがそっ歳します。",  # noqa
    ##################################################
    "## Basic unbinding usage": "## 基本的なイベント解除処理の使い方",
    ##################################################
    "Each `DisplayObject` instance has the `unbind_<event_name>` interfaces, for example, `unbind_click` or `unbind_mousedown` or something else.": "`DisplayObject`の各インスタンスは`unbind_click`や`unbind_mousedown`などの`unbind_<event_name>`という名前の形式のインターフェイスを持っています。",  # noqa
    ##################################################
    "These interfaces can unbind the single handler setting (remove binding setting).": "これらのインターフェイスではイベントハンドラの設定単体を解除することができます。",  # noqa
    ##################################################
    "For example, the following code unbinds the click event, so the interface doesn't call the handler function.": "例えば以下のコード例ではクリックイベントを解除しているため、ハンドラの関数は呼ばれなくなります。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\nclass _ColorOptions(TypedDict):\n    color: str\n\n\ndef on_rectangle_click(e: ap.MouseEvent, options: _ColorOptions) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    # Change the clicked rectangle color to the passed color.\n    rectangle: ap.Rectangle = e.this\n    color: ap.String = ap.String(options["color"])\n    rectangle.fill_color = color\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\noptions: _ColorOptions = {"color": "#f0a"}\nrectangle.click(handler=on_rectangle_click, options=options)\n\nrectangle.unbind_click(handler=on_rectangle_click)\n\nap.save_overall_html(dest_dir_path="mouse_event_basic_basic_unbinding_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\nclass _ColorOptions(TypedDict):\n    color: str\n\n\ndef on_rectangle_click(e: ap.MouseEvent, options: _ColorOptions) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    # Change the clicked rectangle color to the passed color.\n    rectangle: ap.Rectangle = e.this\n    color: ap.String = ap.String(options["color"])\n    rectangle.fill_color = color\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\noptions: _ColorOptions = {"color": "#f0a"}\nrectangle.click(handler=on_rectangle_click, options=options)\n\nrectangle.unbind_click(handler=on_rectangle_click)\n\nap.save_overall_html(dest_dir_path="mouse_event_basic_basic_unbinding_usage/")\n```',  # noqa
    ##################################################
    "When you click the following rectangle, nothing happens.": "以下の四角をクリックしてみても何も起こらないことが確認できます。",  # noqa
    ##################################################
    "## Unbind all event handlers": "## 全てのイベントハンドラの設定を解除する",
    ##################################################
    "Sometimes, it is helpful to unbind specific all the events at once. For example, each event interface has the `unbind_<event_name>_all` method (e.g., `unbind_click_all`). It can unbind all event handlers from that instance.": "特定のイベントの設定を一括で解除するのが役立つ時があります。イベントが設定できる各インスタンスは`unbind_click_all`などの`unbind_<event_name>_all`という名前の形式のインターフェイスを持っており、それを使ってインスタンスから一通りのイベントのハンドラ設定を解除することができます。",  # noqa
    ##################################################
    "The following code calls the `unbind_click_all` method and removes all handler settings.": "以下のコード例では`unbind_click_all`メソッドを呼んで全てのクリックイベントのハンドラ設定を解除しています。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\nclass _ColorOptions(TypedDict):\n    color: str\n\n\ndef change_color_on_rectangle_click(e: ap.MouseEvent, options: _ColorOptions) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle: ap.Rectangle = e.this\n    color: ap.String = ap.String(options["color"])\n    rectangle.fill_color = color\n\n\ndef change_x_on_rectangle_click(e: ap.MouseEvent, options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.x += 50\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\noptions: _ColorOptions = {"color": "#f0a"}\nrectangle.click(handler=change_color_on_rectangle_click, options=options)\nrectangle.click(handler=change_x_on_rectangle_click)\n\nrectangle.unbind_click_all()\n\nap.save_overall_html(dest_dir_path="mouse_event_basic_unbind_all_event_handlers/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\nclass _ColorOptions(TypedDict):\n    color: str\n\n\ndef change_color_on_rectangle_click(e: ap.MouseEvent, options: _ColorOptions) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle: ap.Rectangle = e.this\n    color: ap.String = ap.String(options["color"])\n    rectangle.fill_color = color\n\n\ndef change_x_on_rectangle_click(e: ap.MouseEvent, options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.x += 50\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\noptions: _ColorOptions = {"color": "#f0a"}\nrectangle.click(handler=change_color_on_rectangle_click, options=options)\nrectangle.click(handler=change_x_on_rectangle_click)\n\nrectangle.unbind_click_all()\n\nap.save_overall_html(dest_dir_path="mouse_event_basic_unbind_all_event_handlers/")\n```',  # noqa
    ##################################################
    "Nothing happens when clicking the rectangle (no color change and no x-coordinate change).": "四角をクリックしてみても色の変化やX座標の更新などが発生しないことを確認できます。",  # noqa
    ##################################################
    "## Handler argument names and types": "## ハンドラの引数の名前と型",
    ##################################################
    "Handler function (or method) first argument requires the type of the `MouseEvent`\\.": "ハンドラの関数（もしくはメソッド）の第一引数は`MouseEvent`型の引数が必要になります。",  # noqa
    ##################################################
    "Also, a second argument name is required to be `options`\\. This argument type becomes `dict`\\. If you skip options argument specification at binding the event, then this argument becomes a blank dictionary (`{}`).": "また、第二引数には`options`という名前の辞書の引数が必要になります。イベント設定時にこのoptionsパラメーターの指定を省略した場合にはこの引数の値は空の辞書（`{}`）になります。",  # noqa
    ##################################################
    "## MouseEvent this attribute": "## MouseEvent クラスの this 属性",
    ##################################################
    "The `MouseEvent` instance has the `this` attribute, which becomes an event target instance. So, if you bind the click event to the rectangle instance, the `this` attribute becomes that rectangle instance.": "`MouseEvent`クラスのインスタンスはイベント登録対象のインスタンスとなる`this`属性を持っています。例えばクリックイベントを四角のインスタンスに設定した場合、`this`属性はその四角のインスタンスになります。",  # noqa
    ##################################################
    "## MouseEvent generic type settings": "## MouseEvent クラスのジェネリック型の設定",
    ##################################################
    "Suppose you know that you only use one of the handlers by an instance of a particular type. In that case, you can set generic type settings to the `MouseEvent` type annotation (e.g., `MouseEvent[Rectangle]`).": "もしもハンドラを特定の型のインスタンスのみで使うことが分かっている場合、`MouseEvent`の型アノテーションでジェネリックの型の指定を行うことができます（例: `MouseEvent[Rectangle]`）。",  # noqa
    ##################################################
    "This setting is helpful to determine the `this` attribute type, and the type-checking library, such as the `mypy` or `Pylance`\\, checks the instance type.": "この設定は`this`属性の型の決定に使われ、`mypy`や`Pylance`などの型チェックのライブラリを使っている場合役立つことがあります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=200, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\ndef on_rectangle_mousedown(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mousedown.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle = e.this\n    rectangle.x += 50\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.mousedown(handler=on_rectangle_mousedown)\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=200, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\ndef on_rectangle_mousedown(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mousedown.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle = e.this\n    rectangle.x += 50\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.mousedown(handler=on_rectangle_mousedown)\n```',  # noqa
    ##################################################
    "## MouseEvent stage_x and stage_y attributes": "## MouseEvent クラスの stage_x と stage_y 属性",  # noqa
    ##################################################
    "MouseEvent instance has the `stage_x` and `stage_y` attributes. These attributes are absolute coordinates from the upper-left position of the stage.": "MouseEventクラスのインスタンスは`stage_x`や`stage_y`の各属性のインターフェイスを持っています。これらの属性はステージの左上の位置を基準とした絶対座標となります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=200, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\ndef on_mousemove(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mousemove.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    ap.trace("stage_x:", e.stage_x, "stage_y:", e.stage_y)\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=100, width=50, height=50)\nrectangle.mousemove(handler=on_mousemove)\n\nap.save_overall_html(dest_dir_path="mouse_event_basic_stage_x_and_stage_y")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=200, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\ndef on_mousemove(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mousemove.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    ap.trace("stage_x:", e.stage_x, "stage_y:", e.stage_y)\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=100, width=50, height=50)\nrectangle.mousemove(handler=on_mousemove)\n\nap.save_overall_html(dest_dir_path="mouse_event_basic_stage_x_and_stage_y")\n```',  # noqa
    ##################################################
    "If you open the DevTools console on Chrome (press F12) and move the mouse cursor on the following rectangle, you can check the `stage_x` and `stage_y` coordinates. The previous code positions the rectangle at `(50, 100)`, so the `stage_x` becomes the range of 50 to 100, and `stage_y` becomes 100 to 150.": "F12を押してChromeなどのDevToolsを開き、以下の四角の上でマウスカーソルを動かすと`stage_x`や`stage_y`の座標値を家訓することができます。前述のコードでは四角を`(50, 100)`の位置に設定しているため、`stage_x`の値は50～100の範囲の値となり、`stage_y`の値は100～150の範囲の値となります。",  # noqa
    ##################################################
    "## MouseEvent local_x and local_y attributes": "## MouseEvent クラスの local_x と local_y 属性",  # noqa
    ##################################################
    "MouseEvent instance also has `local_x` and `local_y` attributes. These attributes are the local coordinates from the event registered instance position.": "MouseEventのインスタンスは`local_x`と`local_y`という属性も持っています。これらの属性はイベントが登録されたインスタンスを基準とした相対座標となります。",  # noqa
    ##################################################
    "The following example shows that local_x and local_y become the coordinates in the rectangle area. Both of the `local_x` and `local_y` become a range of 0 to 50 because the rectangle size is 50-pixel.": "以下のコード例ではlocal_xとlocal_yの座標が四角の範囲の座標になっていることを確認できます。四角のサイズが50pxなため、`local_x`と`local_y`の値は両方とも0～50の範囲になります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\ndef on_mousemove(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mousemove.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    ap.trace("local_x:", e.local_x, "local_y:", e.local_y)\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.mousemove(handler=on_mousemove)\n\nap.save_overall_html(dest_dir_path="mouse_event_basic_local_x_and_local_y")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=150, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\n\ndef on_mousemove(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mousemove.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    ap.trace("local_x:", e.local_x, "local_y:", e.local_y)\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.mousemove(handler=on_mousemove)\n\nap.save_overall_html(dest_dir_path="mouse_event_basic_local_x_and_local_y")\n```',  # noqa
    ##################################################
    "Please check on Chrome DevTools (press F12) and move the mouse cursor on the following rectangle.": "F12を押してChromeなどのDevToolsを開き、以下の四角の上でマウスカーソルを動かしてみて出力結果を確認してみてください。",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [click interface](click.md)": "- [click インターフェイス](jp_click.md)",
    ##################################################
    "- [mousedown and mouseup interfaces](mousedown_and_mouseup.md)": "- [mousedown と mouseup のインターフェイス](jp_mousedown_and_mouseup.md)",  # noqa
    ##################################################
    "- [mouseover and mouseout interfaces](mouseover_and_mouseout.md)": "- [mouseover と mouseout のインターフェイス](jp_mouseover_and_mouseout.md)",  # noqa
    ##################################################
    "- [mousemove interface](mousemove.md)": "- [mousemove インターフェイス](jp_mousemove.md)",
    ##################################################
    "## stage_x property API": "## stage_x 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get the x-coordinate of the stage reference.<hr>": "ステージ基準のX座標を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `x`: Int": "- `x`: Int",
    ##################################################
    "  - x-coordinate.": "  - X座標。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_mousedown(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     stage_x: ap.Int = e.stage_x\n...     # Do something here with the coordinate.\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mousedown(on_mousedown)\n```': '```py\n>>> import apysc as ap\n>>> def on_mousedown(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     stage_x: ap.Int = e.stage_x\n...     # Do something here with the coordinate.\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mousedown(on_mousedown)\n```',  # noqa
    ##################################################
    "## stage_y property API": "## stage_y 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get the y-coordinate of the stage reference.<hr>": "ステージ基準のY座標を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `y`: Int": "- `y`: Int",
    ##################################################
    "  - y-coordinate.": "  - Y座標。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_mousedown(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     stage_y: ap.Int = e.stage_y\n...     # Do something here with the coordinate.\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mousedown(on_mousedown)\n```': '```py\n>>> import apysc as ap\n>>> def on_mousedown(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     stage_y: ap.Int = e.stage_y\n...     # Do something here with the coordinate.\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mousedown(on_mousedown)\n```',  # noqa
    ##################################################
    "## local_x property API": "## local_x 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a local x-coordinate event listening instance. For example, this value becomes x-coordinate from Sprite's left-end position by clicking a Sprite instance.<hr>": "イベントが設定されているインスタンス内の相対座標のX座標を取得します。例えばSpriteのインスタンスをクリックした場合にはSpriteの左上の位置を基準とした座標になります。<hr>",  # noqa
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `x`: Int": "- `x`: Int",
    ##################################################
    "  - x-coordinate.": "  - X座標。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_mousedown(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     local_x: ap.Int = e.local_x\n...     # Do something here with the coordinate.\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mousedown(on_mousedown)\n```': '```py\n>>> import apysc as ap\n>>> def on_mousedown(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     local_x: ap.Int = e.local_x\n...     # Do something here with the coordinate.\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mousedown(on_mousedown)\n```',  # noqa
    ##################################################
    "## local_y property API": "## local_y 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get the local y-coordinate of the event listening instance. For example, this value becomes y-coordinate from Sprite's top-end position by clicking a Sprite instance.<hr>": "イベントが設定されているインスタンスないの相対座標のY座標を取得します。例えばSpriteのインスタンスをクリックした場合にはSpriteの左上の位置を基準とした座標になります。<hr>",  # noqa
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `y`: Int": "- `y`: Int",
    ##################################################
    "  - y-coordinate.": "  - Y座標。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_mousedown(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     local_y: ap.Int = e.local_y\n...     # Do something here with the coordinate.\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mousedown(on_mousedown)\n```': '```py\n>>> import apysc as ap\n>>> def on_mousedown(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     local_y: ap.Int = e.local_y\n...     # Do something here with the coordinate.\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mousedown(on_mousedown)\n```',  # noqa
    ##################################################
    "## this property API": "## this 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get an instance of listening to this event.<hr>": "このイベントが設定されているインスタンスを取得します。<hr>",  # noqa
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `this`: VariableNameMixIn": "- `this`: VariableNameMixIn",
    ##################################################
    "  - Instance that listening this event.": "  - このイベントが設定されているインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_custom_event(e: ap.Event[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String("#f0a")\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> e: ap.Event = ap.Event(this=rectangle)\n>>> _ = rectangle.bind_custom_event(\n...     custom_event_type="my_custom_event", handler=on_custom_event, e=e\n... )\n>>> # Do something here and then trigger the custom event\n>>> rectangle.trigger_custom_event(custom_event_type="my_custom_event")\n```': '```py\n>>> import apysc as ap\n>>> def on_custom_event(e: ap.Event[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String("#f0a")\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> e: ap.Event = ap.Event(this=rectangle)\n>>> _ = rectangle.bind_custom_event(\n...     custom_event_type="my_custom_event", handler=on_custom_event, e=e\n... )\n>>> # Do something here and then trigger the custom event\n>>> rectangle.trigger_custom_event(custom_event_type="my_custom_event")\n```',  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: animation_width_and_height.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# animation_width and animation_height interfaces": "# animation_width と animation_height のインターフェイス",  # noqa
    ##################################################
    "This page explains the `animation_width` and `animation_height` interfaces.": "このページでは`animation_width`と`animation_height`メソッドの各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `animation_width` method interface creates an `AnimationWidth` instance. You can animate object width with it.": "`animation_width`インターフェイスは`AnimationWidth`クラスのインスタンスを生成します。そのインスタンスを使って幅のアニメーションを設定することができます。",  # noqa
    ##################################################
    "Similarly, the `animation_height` method interface creates an `AnimationHeight` instance. You can animate object height with it.": "同様に`animation_height`メソッドのインターフェイスは`AnimationHeight`クラスのインスタンスを生成します。そのインスタンスを使って高さのアニメーションを設定することができます。",  # noqa
    ##################################################
    "These interfaces exist on some `DisplayObject` instances, such as the `Rectangle` class.": "これらの各インターフェイスは`Rectangle`クラスなどの`DisplayObject`の各サブクラス上に存在します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The following example sets the width animation (from 50 to 100) with the `animation_width` method:": "以下のコード例では`animation_width`メソッドを使って幅（50から100）のアニメーションを設定しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nDURATION: int = 1000\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_width: ap.AnimationWidth = rectangle.animation_width(\n        width=50, duration=DURATION, easing=EASING\n    )\n    animation_width.animation_complete(on_animation_complete_2)\n    animation_width.start()\n\n\ndef on_animation_complete_2(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_width: ap.AnimationWidth = rectangle.animation_width(\n        width=100, duration=DURATION, easing=EASING\n    )\n    animation_width.animation_complete(on_animation_complete_1)\n    animation_width.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#00aaff")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nanimation_width: ap.AnimationWidth = rectangle.animation_width(\n    width=100, duration=DURATION, easing=EASING\n)\nanimation_width.animation_complete(on_animation_complete_1)\nanimation_width.start()\n\nap.save_overall_html(dest_dir_path="./animation_width_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nDURATION: int = 1000\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_width: ap.AnimationWidth = rectangle.animation_width(\n        width=50, duration=DURATION, easing=EASING\n    )\n    animation_width.animation_complete(on_animation_complete_2)\n    animation_width.start()\n\n\ndef on_animation_complete_2(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_width: ap.AnimationWidth = rectangle.animation_width(\n        width=100, duration=DURATION, easing=EASING\n    )\n    animation_width.animation_complete(on_animation_complete_1)\n    animation_width.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#00aaff")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nanimation_width: ap.AnimationWidth = rectangle.animation_width(\n    width=100, duration=DURATION, easing=EASING\n)\nanimation_width.animation_complete(on_animation_complete_1)\nanimation_width.start()\n\nap.save_overall_html(dest_dir_path="./animation_width_basic_usage/")\n```',  # noqa
    ##################################################
    "Similarly, the following example animates the height with the `animation_height` method:": "同様に以下のコード例では`animation_height`メソッドを使って高さのアニメーションを設定しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nDURATION: int = 1000\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_height: ap.AnimationHeight = rectangle.animation_height(\n        height=50, duration=DURATION, easing=EASING\n    )\n    animation_height.animation_complete(on_animation_complete_2)\n    animation_height.start()\n\n\ndef on_animation_complete_2(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_height: ap.AnimationHeight = rectangle.animation_height(\n        height=100, duration=DURATION, easing=EASING\n    )\n    animation_height.animation_complete(on_animation_complete_1)\n    animation_height.start()\n\n\nap.Stage(\n    stage_width=150, stage_height=200, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#00aaff")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nanimation_height: ap.AnimationHeight = rectangle.animation_height(\n    height=100, duration=DURATION, easing=EASING\n)\nanimation_height.animation_complete(on_animation_complete_1)\nanimation_height.start()\n\nap.save_overall_html(dest_dir_path="./animation_height_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nDURATION: int = 1000\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_height: ap.AnimationHeight = rectangle.animation_height(\n        height=50, duration=DURATION, easing=EASING\n    )\n    animation_height.animation_complete(on_animation_complete_2)\n    animation_height.start()\n\n\ndef on_animation_complete_2(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_height: ap.AnimationHeight = rectangle.animation_height(\n        height=100, duration=DURATION, easing=EASING\n    )\n    animation_height.animation_complete(on_animation_complete_1)\n    animation_height.start()\n\n\nap.Stage(\n    stage_width=150, stage_height=200, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#00aaff")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nanimation_height: ap.AnimationHeight = rectangle.animation_height(\n    height=100, duration=DURATION, easing=EASING\n)\nanimation_height.animation_complete(on_animation_complete_1)\nanimation_height.start()\n\nap.save_overall_html(dest_dir_path="./animation_height_basic_usage/")\n```',  # noqa
    ##################################################
    "## Notes for the Ellipse instance": "## Ellipse のインスタンスにおける特記事項",
    ##################################################
    "The ellipse instance's `animation_width` and `animation_height` interfaces will return an `AnimationWidthForEllipse` and `AnimationHeightForEllipse` instance instead of an `AnimationWidth` instance, for the reason of the internal implementation, as follows:": "楕円のインスタンス（`Ellipse`クラス）の`animation_width`と`animation_height`の各インターフェイスは内部実行が異なる都合で`AnimationWidth`クラスなどの代わりに`以下のコード例のようにAnimationWidthForEllipse`クラスと`AnimationHeightForEllipse`クラスのインスタンスを返却します:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=200, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#00aaff")\nellipse: ap.Ellipse = sprite.graphics.draw_ellipse(x=100, y=100, width=100, height=100)\nanimation_width: ap.AnimationWidthForEllipse = ellipse.animation_width(\n    width=200, duration=1000\n)\nanimation_width.start()\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=200, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#00aaff")\nellipse: ap.Ellipse = sprite.graphics.draw_ellipse(x=100, y=100, width=100, height=100)\nanimation_width: ap.AnimationWidthForEllipse = ellipse.animation_width(\n    width=200, duration=1000\n)\nanimation_width.start()\n```',  # noqa
    ##################################################
    "## animation_width API": "## animation_width API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Set the width animation setting.<hr>": "幅のアニメーションを設定します。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `width`: Int or int": "- `width`: Int or int",
    ##################################################
    "  - The final width of the animation.": "  - 幅のアニメーションの最終値。",
    ##################################################
    "- `duration`: Int or int, default 3000": "- `duration`: Int or int, default 3000",
    ##################################################
    "  - Milliseconds before an animation ends.": "  - アニメーション完了までのミリ秒。",
    ##################################################
    "- `delay`: Int or int, default 0": "- `delay`: Int or int, default 0",
    ##################################################
    "  - Milliseconds before an animation starts.": "  - アニメーション開始までの遅延時間のミリ秒。",
    ##################################################
    "- `easing`: Easing, default Easing.LINEAR": "- `easing`: Easing, default Easing.LINEAR",  # noqa
    ##################################################
    "  - Easing setting.": "  - イージング設定。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `animation_width`: AnimationWidth": "- `animation_width`: AnimationWidth",
    ##################################################
    "  - Created animation setting instance.": "  - 生成されたアニメーションのインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    "To start this animation, you need to call the `start` method of the returned instance.<hr>": "アニメーションを開始するには返却されたインスタンスの`start`メソッドを呼び出す必要があります。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_width(\n...     width=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_width(\n...     width=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [Animation interfaces duration setting](https://simon-ritchie.github.io/apysc/en/animation_duration.html)": "- [各アニメーションインターフェイスの duration （アニメーション時間）設定](https://simon-ritchie.github.io/apysc/jp/jp_animation_duration.html)",  # noqa
    ##################################################
    "- [Animation interfaces delay setting](https://simon-ritchie.github.io/apysc/en/animation_delay.html)": "- [各アニメーションインターフェイスの delay （遅延時間）設定](https://simon-ritchie.github.io/apysc/jp/jp_animation_delay.html)",  # noqa
    ##################################################
    "- [Each animation interface return value](https://simon-ritchie.github.io/apysc/en/animation_return_value.html)": "- [各アニメーションインターフェイスの返却値](https://simon-ritchie.github.io/apysc/jp/jp_animation_return_value.html)",  # noqa
    ##################################################
    "- [Sequential animation setting](https://simon-ritchie.github.io/apysc/en/sequential_animation.html)": "- [連続したアニメーション設定](https://simon-ritchie.github.io/apysc/jp/jp_sequential_animation.html)",  # noqa
    ##################################################
    "- [animation_parallel interface](https://simon-ritchie.github.io/apysc/en/animation_parallel.html)": "- [animation_parallel インターフェイス](https://simon-ritchie.github.io/apysc/jp/jp_animation_parallel.html)",  # noqa
    ##################################################
    "- [Easing enum](https://simon-ritchie.github.io/apysc/en/easing_enum.html)": "- [イージングのenum](https://simon-ritchie.github.io/apysc/jp/jp_easing_enum.html)",  # noqa
    ##################################################
    "## animation_height API": "## animation_height API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Set the height animation setting.<hr>": "高さのアニメーションを設定します。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `height`: Int or int": "- `height`: Int or int",
    ##################################################
    "  - The final height of the animation.": "  - 高さのアニメーションの最終値。",
    ##################################################
    "- `duration`: Int or int, default 3000": "- `duration`: Int or int, default 3000",
    ##################################################
    "  - Milliseconds before an animation ends.": "  - アニメーション完了までのミリ秒。",
    ##################################################
    "- `delay`: Int or int, default 0": "- `delay`: Int or int, default 0",
    ##################################################
    "  - Milliseconds before an animation starts.": "  - アニメーション開始までの遅延時間のミリ秒。",
    ##################################################
    "- `easing`: Easing, default Easing.LINEAR": "- `easing`: Easing, default Easing.LINEAR",  # noqa
    ##################################################
    "  - Easing setting.": "  - イージング設定。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `animation_height`: AnimationHeight": "- `animation_height`: AnimationHeight",
    ##################################################
    "  - Created animation setting instance.": "  - 生成されたアニメーションのインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    "To start this animation, you need to call the `start` method of the returned instance.<hr>": "アニメーションを開始するには返却されたインスタンスの`start`メソッドを呼び出す必要があります。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_height(\n...     height=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_height(\n...     height=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [Animation interfaces duration setting](https://simon-ritchie.github.io/apysc/en/animation_duration.html)": "- [各アニメーションインターフェイスの duration （アニメーション時間）設定](https://simon-ritchie.github.io/apysc/jp/jp_animation_duration.html)",  # noqa
    ##################################################
    "- [Animation interfaces delay setting](https://simon-ritchie.github.io/apysc/en/animation_delay.html)": "- [各アニメーションインターフェイスの delay （遅延時間）設定](https://simon-ritchie.github.io/apysc/jp/jp_animation_delay.html)",  # noqa
    ##################################################
    "- [Each animation interface return value](https://simon-ritchie.github.io/apysc/en/animation_return_value.html)": "- [各アニメーションインターフェイスの返却値](https://simon-ritchie.github.io/apysc/jp/jp_animation_return_value.html)",  # noqa
    ##################################################
    "- [Sequential animation setting](https://simon-ritchie.github.io/apysc/en/sequential_animation.html)": "- [連続したアニメーション設定](https://simon-ritchie.github.io/apysc/jp/jp_sequential_animation.html)",  # noqa
    ##################################################
    "- [animation_parallel interface](https://simon-ritchie.github.io/apysc/en/animation_parallel.html)": "- [animation_parallel インターフェイス](https://simon-ritchie.github.io/apysc/jp/jp_animation_parallel.html)",  # noqa
    ##################################################
    "- [Easing enum](https://simon-ritchie.github.io/apysc/en/easing_enum.html)": "- [イージングのenum](https://simon-ritchie.github.io/apysc/jp/jp_easing_enum.html)",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: array_insert_and_insert_at.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Array class insert and insert_at interfaces": "# Array クラスの insert と insert_at のインターフェイス",  # noqa
    ##################################################
    "This page explains the `Array` class `insert` and `insert_at` method interfaces.": "このページでは`Array`クラスの`insert`と`insert_at`メソッドの各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `insert` and `insert_at` method interfaces append any value at the specified index. Both interfaces behave the same way (the `insert` is the alias of the `insert_at`).": "`insert`と`insert_at`メソッドの各インターフェイスは任意の値を配列の指定されたインデックスへと追加します。それぞれの員スターフェイスは同じ挙動をします（`insert`メソッドは`insert_at`メソッドのエイリアスとなっています）。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `insert` and `insert_at` have the same argument, the `index` and `value`\\. The `index` argument accepts an `int` and `Int` value.": "`insert`と`insert_at`メソッドは`index`と`value`の同じ引数を持っています。`index`引数はPythonビルトインの`int`とapyscの`Int`クラスの値を受け付けます。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 4])\narr.insert(index=1, value=2)\nassert arr == [1, 2, 4]\n\nindex: ap.Int = ap.Int(2)\narr.insert_at(index=index, value=3)\nassert arr == [1, 2, 3, 4]\n```": "```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 4])\narr.insert(index=1, value=2)\nassert arr == [1, 2, 4]\n\nindex: ap.Int = ap.Int(2)\narr.insert_at(index=index, value=3)\nassert arr == [1, 2, 3, 4]\n```",  # noqa
    ##################################################
    "## insert API": "## insert API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Insert value to this array at a specified index. This interface behaves in the same `insert_at` method.<hr>": "この配列の指定されたインデックスの位置へ値を挿入します。このインターフェイスは`insert_at`メソッドと同じ挙動をします。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `index`: Int or int": "- `index`: Int or int",
    ##################################################
    "  - Index to append value.": "  - 値を追加するインデックス。",
    ##################################################
    "- `value`: *": "- `value`: *",
    ##################################################
    "  - Any value to append.": "  - 追加対象の任意の値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 3])\n>>> arr.insert(index=1, value=2)\n>>> arr\nArray([1, 2, 3])\n```": "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 3])\n>>> arr.insert(index=1, value=2)\n>>> arr\nArray([1, 2, 3])\n```",  # noqa
    ##################################################
    "## insert_at API": "## insert_at API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Insert value to this array at a specified index. This interface behaves in the same `insert` method.<hr>": "この配列の指定されたインデックスの位置へ値を挿入します。このインターフェイスは`insert`メソッドと同じ挙動をします。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `index`: Int or int": "- `index`: Int or int",
    ##################################################
    "  - Index to append value.": "  - 値を追加するインデックス。",
    ##################################################
    "- `value`: *": "- `value`: *",
    ##################################################
    "  - Any value to append.": "  - 追加対象の任意の値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 3])\n>>> arr.insert_at(index=1, value=2)\n>>> arr\nArray([1, 2, 3])\n```": "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 3])\n>>> arr.insert_at(index=1, value=2)\n>>> arr\nArray([1, 2, 3])\n```",  # noqa
}



"""This module is for the translation conversion
of the documents.
"""

import os
import re
from typing import Dict
from typing import List
from typing import Match
from typing import Optional
from typing import Pattern
from typing import Union

from apysc._lint_and_doc.docs_lang import Lang


class _TranslationMappingNotFound(Exception):
    pass


_GITHUB_ISSUE_URL: str = "https://github.com/simon-ritchie/apysc/issues"

_EACH_LANG_HEADING_INFO_FORMAT: Dict[Lang, str] = {
    Lang.JP: '<span class="inconspicuous-txt">※この翻訳ドキュメントはスクリプト'
    "によって出力・同期されています。内容が怪しそうな場合は"
    f'<a href="{_GITHUB_ISSUE_URL}" target="_blank">GitHubに'
    "issue</a>を追加したり[英語の原文]({source_doc_path})の"
    "確認をお願いします。</span>",
}

_EACH_LANG_API_INTERFACE_SIGNATURE_MAPPING: Dict[Lang, str] = {
    Lang.JP: "インターフェイスの構造",
}

_API_SIGNATURE_PATTERN: Pattern = re.compile(
    pattern=(r"^\*\*\[Interface signature\]\*\*" r"(?P<tail_str>.*?)$")
)


def apply_translation_to_doc(*, md_file_path: str, lang: Lang) -> str:
    """
    Apply a translation mapping to a specified markdown
    document. This interface saves a translated file
    with a file name with a language suffix (e.g.,
    jp_your_doc.md).

    Parameters
    ----------
    md_file_path : str
        A source markdown file path.
    lang : Lang
        A target language setting.

    Returns
    -------
    translated_file_path : str
        A translated document file path.
    """
    from apysc._file import file_util
    from apysc._lint_and_doc import translation_mapping_utils
    from apysc._lint_and_doc.document_text_split_util import BodyText
    from apysc._lint_and_doc.document_text_split_util import CodeBlock
    from apysc._lint_and_doc.document_text_split_util import Heading
    from apysc._lint_and_doc.document_text_split_util import split_markdown_document

    markdown: str = file_util.read_txt(file_path=md_file_path)
    splitted_values: List[
        Union[Heading, BodyText, CodeBlock]
    ] = split_markdown_document(markdown_txt=markdown)
    mapping_data: Dict[str, str] = translation_mapping_utils.read_mapping_data(
        src_doc_file_path=md_file_path, lang=lang
    )
    keys: List[str] = translation_mapping_utils.convert_splitted_values_to_keys(
        splitted_values=splitted_values
    )
    keys = translation_mapping_utils.remove_empty_keys(keys=keys)
    translated_doc: str = ""
    translated_doc = _add_heading_info_if_exists(
        translated_doc=translated_doc, lang=lang, md_file_path=md_file_path
    )
    for key in keys:
        key_: str = translation_mapping_utils.remove_escaping_from_key_or_value(
            key_or_val=key
        )
        if translation_mapping_utils.is_translation_skipping_key(key=key_):
            continue
        translated_doc = _append_double_line_breaks_if_txt_is_not_blank(
            txt=translated_doc
        )

        is_mapping_unnecessary_key: bool = (
            translation_mapping_utils.is_mapping_unnecessary_key(key=key_)
        )
        if is_mapping_unnecessary_key:
            key_ = _apply_mapping_if_translated_str_is_api_sig(
                translated_str=key_, lang=lang
            )
            translated_doc += key_
            continue

        translated_str: str = mapping_data.get(key_, "")
        translated_doc += translated_str
        _validate_translated_str_is_not_blank(
            translated_str=translated_str, key=key, md_file_path=md_file_path
        )
        _validate_sharp_heading_symbol_num_are_same(
            translated_str=translated_str, key=key, md_file_path=md_file_path
        )
        _validate_first_spaces_nums_are_same(
            translated_str=translated_str, key=key, md_file_path=md_file_path
        )
        _validate_markdown_list_hyphen_symbols_are_same(
            translated_str=translated_str, key=key, md_file_path=md_file_path
        )
        _validate_tail_hr_tag(
            translated_str=translated_str, key=key, md_file_path=md_file_path
        )
        _validate_first_br_tags_and_list_symbols_are_same(
            translated_str=translated_str, key=key, md_file_path=md_file_path
        )
        _validate_first_full_width_list_symbols_are_same(
            translated_str=translated_str, key=key, md_file_path=md_file_path
        )

    translated_file_path: str = (
        translation_mapping_utils.get_translated_file_path_from_src_path(
            source_doc_path=md_file_path, lang=lang
        )
    )
    translated_doc = _remove_unnecessary_line_break_between_list(
        translated_doc=translated_doc
    )
    translated_doc = _remove_line_break_between_api_docs_list_br_tag(
        translated_doc=translated_doc
    )
    file_util.save_plain_txt(txt=translated_doc, file_path=translated_file_path)

    return translated_file_path


def _append_double_line_breaks_if_txt_is_not_blank(*, txt: str) -> str:
    """
    Append double line breaks if a specified text is not
    a blank string.

    Parameters
    ----------
    txt : str
        A target text.

    Returns
    -------
    txt : str
        A result text.
    """
    if txt != "":
        txt += "\n\n"
    return txt


class _FirstFullWidthListSymbolsAreNotSame(Exception):
    pass


def _validate_first_full_width_list_symbols_are_same(
    *, translated_str: str, key: str, md_file_path: str
) -> None:
    """
    Validate whether specified first full-width list symbols
    are the same or not.

    Parameters
    ----------
    translated_str : str
        A translated string.
    key : str
        A key (source) string.
    md_file_path : str
        A source markdown file path.

    Raises
    ------
    _FirstFullWidthListSymbolsAreNotSame
        If specified strings' first full-width list symbols
        are not the same.
    """
    if not key.strip().startswith("・"):
        return
    if translated_str.strip().startswith("・"):
        return
    raise _FirstFullWidthListSymbolsAreNotSame(
        "Specified strings' first full-width list symbols "
        "are not the same:"
        f"\nSource string: {key}"
        f"\nTranslated string: {translated_str}"
        f"\nSource document path: {md_file_path}"
    )


class _BrTagsAndListSymbolsAreNotSame(Exception):
    pass


def _validate_first_br_tags_and_list_symbols_are_same(
    *, translated_str: str, key: str, md_file_path: str
) -> None:
    """
    Validate whether a string's first break tags and list
    symbols（・） are the same between a source string and
    a key string.

    Parameters
    ----------
    translated_str : str
        A translated string.
    key : str
        A key (source) string.
    md_file_path : str
        A source markdown file path.

    Raises
    ------
    _BrTagsAndListSymbolsAreNotSame
        This interface makes an exception if specified strings'
        first break tags and list symbols are not the same.
    """
    if not key.startswith("<br> ・"):
        return
    if translated_str.startswith("<br> ・"):
        return
    raise _BrTagsAndListSymbolsAreNotSame(
        "First break tags and list symbols are not the same:"
        f"\nSource string: {key}"
        f"\nTranslated string: {translated_str}"
        f"\nSource document path: {md_file_path}"
    )


def _remove_line_break_between_api_docs_list_br_tag(*, translated_doc: str) -> str:
    """
    Remove a line break between an API docs list's break tag.

    Parameters
    ----------
    translated_doc : str
        A translated string.

    Returns
    -------
    translated_doc : str
        A result string.
    """
    while "\n<br> ・" in translated_doc:
        translated_doc = translated_doc.replace("\n<br> ・", "<br> ・")
    return translated_doc


class _InvalidTailsHrTag(Exception):
    pass


def _validate_tail_hr_tag(*, translated_str: str, key: str, md_file_path: str) -> None:
    """
    Validate whether a tail of translated string's `<hr>`
    tag is valid or not.

    Parameters
    ----------
    translated_str : str
        A translated string.
    key : str
        A key (source) string.
    md_file_path : str
        A source markdown file path.

    Raises
    ------
    _InvalidTailsHrTag
        This interface makes an exception if a tail of
        a source text is the `<hr>` tag and a translated
        string is not.
    """
    if not key.endswith("<hr>"):
        return
    if translated_str.endswith("<hr>"):
        return
    raise _InvalidTailsHrTag(
        "End of a translated string is not the `<hr>` tag."
        f"\nSource string: {key}"
        f"\nTranslated string: {translated_str}"
        f"\nSource document path: {md_file_path}"
    )


class _MarkdownListHyphenSymbolsAreNotSame(Exception):
    pass


def _validate_markdown_list_hyphen_symbols_are_same(
    *, translated_str: str, key: str, md_file_path: str
) -> None:
    """
    Validate whether markdown's first hyphen symbols are the
    same or not.

    Parameters
    ----------
    translated_str : str
        A translated string.
    key : str
        A key (source) string.
    md_file_path : str
        A source markdown file path.

    Raises
    ------
    _MarkdownListHyphenSymbolsAreNotSame
        This interface makes an exception if markdown's first
        hyphen symbols are not the same.
    """
    if not key.strip().startswith("- "):
        return
    if translated_str.strip().startswith("- "):
        return
    raise _MarkdownListHyphenSymbolsAreNotSame(
        "First character of list's hyphen symbols are not the same."
        f"\nSource string: {key}"
        f"\nTranslated string: {translated_str}"
        f"\nSource document path: {md_file_path}"
    )


class _FirstSpacesNumAreDifferent(Exception):
    pass


def _validate_first_spaces_nums_are_same(
    *, translated_str: str, key: str, md_file_path: str
) -> None:
    """
    Validate whether first spaces numbers are the same between
    a source markdown and a translated markdown text.

    Parameters
    ----------
    translated_str : str
        A translated string.
    key : str
        A key (source) string.
    md_file_path : str
        A source markdown file path.

    Raises
    ------
    _FirstSpacesNumAreDifferent
        This interface raises an exception if first spaces
        numbers are different.
    """
    translated_str_first_spaces_num: int = _get_first_spaces_num(txt=translated_str)
    key_first_spaces_num: int = _get_first_spaces_num(txt=key)
    if translated_str_first_spaces_num == key_first_spaces_num:
        return
    raise _FirstSpacesNumAreDifferent(
        "First spaces numbers are not the same between a "
        "source markdown and a translated markdown text."
        f"\nSource string: {key}"
        f"\nTranslated string: {translated_str}"
        f"\nSource document path: {md_file_path}"
    )


def _get_first_spaces_num(*, txt: str) -> int:
    """
    Get a first spaces number of a specified string.

    Parameters
    ----------
    txt : str
        A target string.

    Returns
    -------
    first_spaces_num : int
        A first spaces number.
    """
    first_spaces_num: int = 0
    for char in txt:
        if char != " ":
            break
        first_spaces_num += 1
    return first_spaces_num


def _remove_unnecessary_line_break_between_list(*, translated_doc: str) -> str:
    """
    Remove an unnecessary line break between each markdown's list.

    Parameters
    ----------
    translated_doc : str
        A target translated document to remove an unnecessary line break.

    Returns
    -------
    result_translated_doc : str
        A target translated document to remove an unnecessary
        line break.
    """
    result_translated_doc: str = re.sub(
        pattern=(r"(?P<prev_part>\s*?- .+?$\n)\n(?P<after_part>\s*?- )"),
        repl=r"\g<prev_part>\g<after_part>",
        string=translated_doc,
        flags=re.MULTILINE,
    )
    return result_translated_doc


class _InvalidHeadingSharpSymbolNumber(Exception):
    pass


def _validate_sharp_heading_symbol_num_are_same(
    *, translated_str: str, key: str, md_file_path: str
) -> None:
    """
    Validate whether a sharp heading symbol of source and
    translated string are the same.

    Parameters
    ----------
    translated_str : str
        A translated string.
    key : str
        A key (source) string.
    md_file_path : str
        A source markdown file path.

    Raises
    ------
    _InvalidHeadingSharpSymbolNumber
        The symbol number is different for a specified translated
        string and a source string's sharp symbol heading.
    """
    translated_str_sharp_symbol_num: int = _get_sharp_heading_symbol_num(
        target_str=translated_str
    )
    key_str_sharp_symbol_num: int = _get_sharp_heading_symbol_num(target_str=key)
    if translated_str_sharp_symbol_num == key_str_sharp_symbol_num:
        return
    raise _InvalidHeadingSharpSymbolNumber(
        "There is a difference between source document and "
        "translated document's sharp heading symbol number."
        f"\n\nSource string: {key}"
        f"\nTranslated string: {translated_str}"
        f"\nSource document path: {md_file_path}"
    )


def _get_sharp_heading_symbol_num(*, target_str: str) -> int:
    """
    Get a sharp heading symbol number from a specified string.

    Parameters
    ----------
    target_str : str
        A target string to check.

    Returns
    -------
    sharp_symbol_num : int
        - A sharp heading symbol number.
    """
    sharp_symbol_num: int = 0
    for char in target_str:
        if char != "#":
            break
        sharp_symbol_num += 1
    return sharp_symbol_num


def _apply_mapping_if_translated_str_is_api_sig(
    *, translated_str: str, lang: Lang
) -> str:
    """
    Apply an API signature translation mapping if a specified
    translated string is a signature string.

    Parameters
    ----------
    translated_str : str
        A target tlanslated string.
    lang : Lang
        A target language.

    Returns
    -------
    translated_str : str
        An applied string. This interface returns an argument
        value if a specified translated string is not a signature
        string.
    """
    sig_label_mapping: str = _EACH_LANG_API_INTERFACE_SIGNATURE_MAPPING.get(lang, "")
    if sig_label_mapping == "":
        return translated_str
    match: Optional[Match] = _API_SIGNATURE_PATTERN.search(string=translated_str)
    if match is None:
        return translated_str
    translated_str = _API_SIGNATURE_PATTERN.sub(
        repl=rf"**[{sig_label_mapping}]**\g<tail_str>", string=translated_str, count=1
    )
    return translated_str


def _add_heading_info_if_exists(
    *, translated_doc: str, lang: Lang, md_file_path: str
) -> str:
    """
    Add a target language's heading information text
    to a specified translated text.

    Parameters
    ----------
    translated_doc : str
        A target translated document's text.
    lang : Lang
        A target language.
    md_file_path : str
        A target source document path.

    Returns
    -------
    translated_doc : str
        A result translated document's text.
        This interface directly returns argument value
        if there is no heading information setting.
    """
    heading_info_format: str = _EACH_LANG_HEADING_INFO_FORMAT.get(lang, "")
    if heading_info_format == "":
        return translated_doc
    source_doc_path: str = os.path.basename(md_file_path)
    source_doc_path = source_doc_path.replace(".md", ".html")
    source_doc_path = f"https://simon-ritchie.github.io/apysc/en/{source_doc_path}"
    heading_info_format = heading_info_format.format(
        source_doc_path=source_doc_path,
    )
    translated_doc += heading_info_format
    return translated_doc


def _validate_translated_str_is_not_blank(
    *, translated_str: str, key: str, md_file_path: str
) -> None:
    """
    Validate whether a translated string is not a blank string.

    Parameters
    ----------
    translated_str : str
        A target translated string.
    key : str
        A target key string (original source text).
    md_file_path : str
        A source markdown file path.

    Raises
    ------
    _TranslationMappingNotFound
        If a specified translated string is a blank string.
    """
    if translated_str != "":
        return
    raise _TranslationMappingNotFound(
        "There is no translation mapping."
        f"\nOriginal source text: {key}"
        f"\nSource document path: {md_file_path}"
    )



"""This module is for the translation mapping data of the
following document:

Document file: timer_reset.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Timer class reset interface": "# Timer クラスの reset インターフェイス",
    ##################################################
    "This page explains the `Timer` class `reset` method interface.": "このページでは`Timer`クラスの`reset`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `reset` method interface resets a timer count and stops it.": "`reset`メソッドのインターフェイスはタイマーのカウントをリセットしそしてタイマーを停止します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `reset` method has no arguments.": "`reset`メソッドは引数を必要としません。",
    ##################################################
    "The following example rotates the rectangle 90 degrees (`repeat_count=90`) by the first-timer and then reset and start the second one. The second-timer resets and restart the first-timer after 1 second.": "以下のコード例では1つ目のタイマーで四角を90度回転させています（`repeat_count=90`）。回転処理が終わったら2つ目のタイマーをリセットさせてからスタートさせています。スタート後に1秒経過したら1つ目のタイマーをリセットしてから再度スタートさせています。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\nclass _TimerOptions(TypedDict):\n    timer: ap.Timer\n\n\ndef on_first_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the first-timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.rotation_around_center += 1\n\n\ndef on_first_timer_complete(e: ap.TimerEvent, options: _TimerOptions) -> None:\n    """\n    The handler that the first-timer calls when completed.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    timer_2: ap.Timer = options["timer"]\n    timer_2.reset()\n    timer_2.start()\n\n\ndef on_second_timer(e: ap.TimerEvent, options: _TimerOptions) -> None:\n    """\n    The handler that the second timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    timer_1: ap.Timer = options["timer"]\n    timer_1.reset()\n    timer_1.start()\n\n\noptions_1: _RectOptions = {"rectangle": rectangle}\ntimer_1: ap.Timer = ap.Timer(\n    handler=on_first_timer, delay=ap.FPS.FPS_60, repeat_count=90, options=options_1\n)\n\noptions_2: _TimerOptions = {"timer": timer_1}\ntimer_2: ap.Timer = ap.Timer(\n    handler=on_second_timer, delay=1000, repeat_count=1, options=options_2\n)\noptions_2 = {"timer": timer_2}\ntimer_1.timer_complete(on_first_timer_complete, options=options_2)\ntimer_1.start()\n\nap.save_overall_html(dest_dir_path="timer_reset_basic_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\nclass _TimerOptions(TypedDict):\n    timer: ap.Timer\n\n\ndef on_first_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the first-timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.rotation_around_center += 1\n\n\ndef on_first_timer_complete(e: ap.TimerEvent, options: _TimerOptions) -> None:\n    """\n    The handler that the first-timer calls when completed.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    timer_2: ap.Timer = options["timer"]\n    timer_2.reset()\n    timer_2.start()\n\n\ndef on_second_timer(e: ap.TimerEvent, options: _TimerOptions) -> None:\n    """\n    The handler that the second timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    timer_1: ap.Timer = options["timer"]\n    timer_1.reset()\n    timer_1.start()\n\n\noptions_1: _RectOptions = {"rectangle": rectangle}\ntimer_1: ap.Timer = ap.Timer(\n    handler=on_first_timer, delay=ap.FPS.FPS_60, repeat_count=90, options=options_1\n)\n\noptions_2: _TimerOptions = {"timer": timer_1}\ntimer_2: ap.Timer = ap.Timer(\n    handler=on_second_timer, delay=1000, repeat_count=1, options=options_2\n)\noptions_2 = {"timer": timer_2}\ntimer_1.timer_complete(on_first_timer_complete, options=options_2)\ntimer_1.start()\n\nap.save_overall_html(dest_dir_path="timer_reset_basic_usage/")\n```',  # noqa
    ##################################################
    "## reset API": "## reset API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Reset the timer count and stop this timer.<hr>": "タイマーのカウントをリセットし、そしてタイマーの停止を行います。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.x += 1\n...     with ap.If(rectangle.x > 100):\n...         timer: ap.Timer = e.this\n...         timer.reset()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> _ = ap.Timer(on_timer, delay=33.3, options=options).start()\n```': '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.x += 1\n...     with ap.If(rectangle.x > 100):\n...         timer: ap.Timer = e.this\n...         timer.reset()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> _ = ap.Timer(on_timer, delay=33.3, options=options).start()\n```',  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: datetime_millisecond.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# DateTime class millisecond property": "# DateTime クラスの millisecond 属性",
    ##################################################
    "This page explains the `DateTime` class's `millisecond` property interface.": "このページでは`DateTime`クラスの`millisecond`属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `millisecond` property gets or sets a millisecond value.": "`millisecond`属性ではミリ秒の値の取得もしくは設定を行うことができます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "A `DateTime` instance has its property interface.": "`DateTime`クラスのインスタンスがその属性のインターフェイスを持っています。",  # noqa
    ##################################################
    "Its getter interface returns a millisecond's `Int` value.": "そのインタ費フェイスのgetterでは`Int`型のミリ秒の値を返却します。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\ndatetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=5, millisecond=500)\nmillisecond: ap.Int = datetime_.millisecond\nassert millisecond == 500\n```": "```py\n# runnable\nimport apysc as ap\n\ndatetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=5, millisecond=500)\nmillisecond: ap.Int = datetime_.millisecond\nassert millisecond == 500\n```",  # noqa
    ##################################################
    "Also, its setter interface accepts a millisecond's `Int` value.": "また、setter側のインターフェイスでは同様に`Int`型の値を受け付けます。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\ndatetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=5, millisecond=500)\ndatetime_.millisecond = ap.Int(300)\nassert datetime_.millisecond == 300\n```": "```py\n# runnable\nimport apysc as ap\n\ndatetime_: ap.DateTime = ap.DateTime(year=2022, month=12, day=5, millisecond=500)\ndatetime_.millisecond = ap.Int(300)\nassert datetime_.millisecond == 300\n```",  # noqa
    ##################################################
    "## millisecond property API": "## millisecond 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a current millisecond value.<hr>": "現在のミリ秒の値を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `millisecond`: Int": "- `millisecond`: Int",
    ##################################################
    "  - A current millisecond value.": "  - 現在のミリ秒の値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> datetime_: ap.DateTime = ap.DateTime(\n...     year=2022, month=12, day=1, millisecond=500\n... )\n>>> datetime_.millisecond\nInt(500)\n\n>>> datetime_.millisecond = ap.Int(300)\n>>> datetime_.millisecond\nInt(300)\n```": "```py\n>>> import apysc as ap\n>>> datetime_: ap.DateTime = ap.DateTime(\n...     year=2022, month=12, day=1, millisecond=500\n... )\n>>> datetime_.millisecond\nInt(500)\n\n>>> datetime_.millisecond = ap.Int(300)\n>>> datetime_.millisecond\nInt(300)\n```",  # noqa
}
